"""
Prueba de concepto:

Este test:

Toma una cola de rabbitmq a la que llegan los datos
Activa la clase que lee el canales

lo envía a

un ciclo

Cada parte aparece en diferentes procesos

El módulo cycle_read se puede reemplazar por una GUI

"""

import asyncio
import concurrent.futures
from functools import partial
from multiprocessing import Manager
from websocketdatamanager.rabbitmq import ReadMQBroker
from networktools.environment import get_env_variable
from datetime import datetime, timedelta
from tasktools.taskloop import TaskLoop
from networktools.colorprint import rprint, bprint, gprint


async def cycle_read(*args, **kwargs):
    delta = kwargs.get("delta", 2)
    sleep = kwargs.get('sleep', .1)
    anterior = kwargs.get('anterior')
    while (ahora := datetime.now()) <= anterior + timedelta(seconds=delta):
        await asyncio.sleep(sleep)
    kwargs["anterior"] = ahora
    queue_status = args[0]
    rprint(f"Desde {anterior} hasta {ahora}")
    if not queue_status.empty():
        for i in range(queue_status.qsize()):
            msg = queue_status.get()
            queue_status.task_done()
            bprint("===RECIBIDO===")
            gprint(msg)
    bprint("===--------===" * 5)
    return args, kwargs


def read_queue(args, kwargs):
    loop = asyncio.get_event_loop()
    kwargs["anterior"] = datetime.now()
    print(f"In read queue {args}, {kwargs}")
    task = TaskLoop(cycle_read, args, kwargs)
    task.create()
    try:
        loop.run_forever()
    except Exception as e:
        raise e


if __name__ == '__main__':
    amqp_data = {
        'amqp': True,
        'code': 'status',
        'vhost': "gpsdata",
        'host': "10.54.217.95",
        'credentials': ("enugeojson_prod", "geognss_prod"),
        'queue_name': 'status_gnss',
        'routing_key': 'status',
        'exchange': 'status_exchange',
        'durable': True,
        'consumer_tag': 'mq2ws',
        "log_path": "./readmqbroker",
        "multiprocessing": True,
        "queue_destiny": "queue2read"
    }
    print(amqp_data)
    workers = 2
    with concurrent.futures.ProcessPoolExecutor(workers) as executor:
        manager = Manager()
        queue_status = manager.Queue()
        set_queue = {"status": queue_status}
        amqp_data['queue2read'] = queue_status
        engine_opts = {"status": amqp_data}
        reader = ReadMQBroker(
            set_queue,
            engine_opts,
        )
        loop = asyncio.get_event_loop()
        loop.run_in_executor(executor, reader.task_cycle)
        loop.run_in_executor(
            executor,
            partial(read_queue, [queue_status], {
                "delta": 10,
                "sleep": .1
            }))
        try:
            loop.run_forever()
        except Exception as e:
            raise e



from data_amqp import AMQPData


class RMQEngine:
    def __init__(self, *args, **kwargs):
        self.__status = False
        self.__queue_name = kwargs.get('queue_destiny')
        self.amqp = None
        if kwargs.get('amqp', False):
            try:
                self.amqp_opt = kwargs['amqp']
                self.amqp = AMQPData(**kwargs)
                self.__status = True
                self.__queue = kwargs.get(self.__queue_name)
                self.__queue_active = False
            except Exception as e:
                self.__status = False
                raise e

    @property
    def status(self):
        return self.__status

    def active_queue_switch(self):
        self.__queue_active = not self.__queue_active

    def enu_data(self, msg):
        self.amqp.manage_json_data(msg)

    def cycle(self):
        loop, task = self.amqp.create_task(self.__queue, self.__queue_active)
        if not loop.is_running():
            loop.run_forever()

    def connect(self):
        if self.amqp:
            self.amqp.connect()



from websocketdatamanager.rmq_engine import RMQEngine
from tasktools.taskloop import TaskLoop
import asyncio


class ReadMQBroker:
    """
    This class pretends to create a knut in what receive data from RMQ queues and send directly to the objects form dj-collector on the database
    """

    def __init__(self, queue_set, engine_opts, step=0.1, *args, **kwargs):
        self.step = step
        self.queue_set = queue_set
        self.engine_opts = engine_opts
        self.engine = False

    async def cycle_read_from_rmq(self, *args, **kwargs):
        engine = args[0]
        v = args[1]
        channels = args[2]
        if v == 0:
            aux_set = set()
            for channel in channels:
                rmq = engine.get(channel)
                try:
                    rmq.connect()
                except Exception as e:
                    print(f'''Error on connection to dj es -> channel {channel},
                          error {e}''')
                    v = 0
                    return [engine, v, channels], {"error": "No se puedo conectar"}
                aux_set.add(channel)
            for channel in aux_set:
                channels.remove(channel)
            aux_set = None
            v += 1
            await asyncio.sleep(self.step)
            return [engine, v, channels], {}
        else:
            for channel, rmq in engine.items():
                try:

                    queue = self.queue_set.get(channel)
                    # breakpoint()
                    queue, active = await rmq.amqp.consume_exchange_mq(queue, True)
                except Exception as e:
                    print("Error en cycle send to dj es->%s" % e)
                    v = 0
                    channels.add(channel)
            await asyncio.sleep(self.step)
            return [engine, v, channels], {}

    def task_cycle(self):
        loop = asyncio.get_event_loop()
        tasks_list = []
        """
        RMQ Task
        """
        rmq_engine = {channel: RMQEngine(**value_dict)
                      for channel, value_dict in self.engine_opts.items()}
        {rmq.active_queue_switch() for rmq in rmq_engine.values()}
        # se crea el conjunto de canales existentes
        channels = set(self.engine_opts.keys())
        # mq 2 ws
        # la args de rmq
        # se activa ciclo de lectura rmq

        cycle_rmq_args = [rmq_engine, 0, channels]
        task = TaskLoop(self.cycle_read_from_rmq,
                        cycle_rmq_args, {},
                        **{"name": "read_from_rabbitmq"})
        task.create()
        if not loop.is_running():
            loop.run_forever()



from setuptools import setup
from pathlib import Path

path = Path(__file__).resolve().parent
with open(path/'README.md', encoding='utf-8') as f:
    long_description = f.read()

with open(path/'VERSION') as version_file:
    version = version_file.read().strip()


setup(name='rmq_engine',
      version=version,
      description='Engine class for AMQP collect data froma a queue',
      url='http://gitlab.csn.uchile.cl/dpineda/rmq_engine',
      author='David Pineda Osorio',
      author_email='dpineda@csn.uchile.cl',
      license='MIT',
      packages=['rmq_engine'],
      keywords = ["amqp","data", "broker", "async",],
      install_requires=["data-amqp"],
      entry_points={
        },
      include_package_data=True,
      long_description=long_description,
      long_description_content_type='text/markdown',
      zip_safe=False)
